@@ -88,6 +88,19 @@ struct guac_user_info {
      * stated resolution of the display size request is recommended.
      */
     int optimal_resolution;
+    
+    /**
+     * The timezone of the remote system.  If the client does not provide
+     * a specific timezone then this will be NULL.  The format of the timezone
+     * is the standard tzdata naming convention.
+     */
+    const char* timezone;
+    
+    /**
+     * The Guacamole protocol version that the remote system supports, allowing
+     * for feature support to be negotiated between client and server.
+     */
+    guac_protocol_version protocol_version;
 
 };
 
@@ -475,6 +488,27 @@ struct guac_user {
      */
     guac_user_audio_handler* audio_handler;
 
+    /**
+     * Handler for argv events (updates to the connection parameters of an
+     * in-progress connection) sent by the Guacamole web-client.
+     *
+     * The handler takes a guac_stream which contains the stream index and
+     * will persist through the duration of the transfer, the mimetype of
+     * the data being transferred, and the argument (connection parameter)
+     * name.
+     *
+     * Example:
+     * @code
+     *     int argv_handler(guac_user* user, guac_stream* stream,
+     *             char* mimetype, char* name);
+     *
+     *     int guac_user_init(guac_user* user, int argc, char** argv) {
+     *         user->argv_handler = argv_handler;
+     *     }
+     * @endcode
+     */
+    guac_user_argv_handler* argv_handler;
+
 };
 
 /**
@@ -519,7 +553,7 @@ int guac_user_handle_connection(guac_user* user, int usec_timeout);
 /**
  * Call the appropriate handler defined by the given user for the given
  * instruction. A comparison is made between the instruction opcode and the
- * initial handler lookup table defined in user-handlers.c. The intial handlers
+ * initial handler lookup table defined in user-handlers.c. The initial handlers
  * will in turn call the user's handler (if defined).
  *
  * @param user
@@ -546,8 +580,12 @@ int guac_user_handle_instruction(guac_user* user, const char* opcode,
  * Allocates a new stream. An arbitrary index is automatically assigned
  * if no previously-allocated stream is available for use.
  *
- * @param user The user to allocate the stream for.
- * @return The next available stream, or a newly allocated stream.
+ * @param user
+ *     The user to allocate the stream for.
+ *
+ * @return
+ *     The next available stream, or a newly allocated stream, or NULL if the
+ *     maximum number of active streams has been reached.
  */
 guac_stream* guac_user_alloc_stream(guac_user* user);
 
@@ -648,6 +686,32 @@ guac_object* guac_user_alloc_object(guac_user* user);
  */
 void guac_user_free_object(guac_user* user, guac_object* object);
 
+/**
+ * Streams the given connection parameter value over an argument value stream
+ * ("argv" instruction), exposing the current value of the named connection
+ * parameter to the given user. The argument value stream will be automatically
+ * allocated and freed.
+ *
+ * @param user
+ *     The Guacamole user who should receive the connection parameter value.
+ *
+ * @param socket
+ *     The socket over which instructions associated with the argument value
+ *     stream should be sent.
+ *
+ * @param mimetype
+ *     The mimetype of the data within the connection parameter value being
+ *     sent.
+ *
+ * @param name
+ *     The name of the connection parameter being sent.
+ *
+ * @param value
+ *     The current value of the connection parameter being sent.
+ */
+void guac_user_stream_argv(guac_user* user, guac_socket* socket,
+        const char* mimetype, const char* name, const char* value);
+
 /**
  * Streams the image data of the given surface over an image stream ("img"
  * instruction) as PNG-encoded data. The image stream will be automatically
@@ -765,6 +829,17 @@ void guac_user_stream_webp(guac_user* user, guac_socket* socket,
         guac_composite_mode mode, const guac_layer* layer, int x, int y,
         cairo_surface_t* surface, int quality, int lossless);
 
+/**
+ * Returns whether the given user supports the "required" instruction.
+ * 
+ * @param user
+ *     The Guacamole user to check for support of the "required" instruction.
+ * 
+ * @return 
+ *     Non-zero if the user supports the "required" instruction, otherwise zero.
+ */
+int guac_user_supports_required(guac_user* user);
+
 /**
  * Returns whether the given user supports WebP. If the user does not
  * support WebP, or the server cannot encode WebP images, zero is returned.
