@@ -155,6 +155,9 @@ int guac_client_init(guac_client* client, int argc, char** argv) {
     pthread_mutexattr_settype(&(rdp_client->attributes),
             PTHREAD_MUTEX_RECURSIVE);
 
+    /* Initalize the lock */
+    pthread_rwlock_init(&(rdp_client->lock), NULL);
+
     /* Set handlers */
     client->join_handler = guac_rdp_user_join_handler;
     client->free_handler = guac_rdp_client_free_handler;
@@ -216,6 +219,8 @@ int guac_rdp_client_free_handler(guac_client* client) {
     if (rdp_client->audio_input != NULL)
         guac_rdp_audio_buffer_free(rdp_client->audio_input);
 
+    pthread_rwlock_destroy(&(rdp_client->lock));
+
     /* Free client data */
     free(rdp_client);
 
