@@ -22,25 +22,39 @@
 
 #include "keymap.h"
 
+#include <freerdp/freerdp.h>
 #include <guacamole/client.h>
 
 /**
- * The current local state of a key - either pressed or released.
+ * The maximum number of distinct keysyms that any particular keyboard may support.
  */
-typedef enum guac_rdp_key_state {
+#define GUAC_RDP_KEYBOARD_MAX_KEYSYMS 1024
+
+/**
+ * The maximum number of unique modifier variations that any particular keysym
+ * may define. For example, on a US English keyboard, an uppercase "A" may be
+ * typed by pressing Shift+A with Caps Lock unset, or by pressing A with Caps
+ * Lock set (two variations).
+ */
+#define GUAC_RDP_KEY_MAX_DEFINITIONS 4
+
+/**
+ * All possible sources of RDP key events tracked by guac_rdp_keyboard.
+ */
+typedef enum guac_rdp_key_source {
 
     /**
-     * The state associated with a key that is released (not currently
-     * pressed / held down).
+     * The key event was received directly from the Guacamole client via a
+     * "key" instruction.
      */
-    GUAC_RDP_KEY_RELEASED = 0,
+    GUAC_RDP_KEY_SOURCE_CLIENT = 0,
 
     /**
-     * The state associated with a key that is currently pressed (held down).
+     * The key event is being synthesized internally within the RDP support.
      */
-    GUAC_RDP_KEY_PRESSED = 1
+    GUAC_RDP_KEY_SOURCE_SYNTHETIC = 1
 
-} guac_rdp_key_state;
+} guac_rdp_key_source;
 
 /**
  * A representation of a single key within the overall local keyboard,
@@ -50,21 +64,31 @@ typedef enum guac_rdp_key_state {
 typedef struct guac_rdp_key {
 
     /**
-     * The definition of this key within the RDP server's keymap (keyboard
-     * layout). This definition describes which scancode corresponds to this
+     * All definitions of this key within the RDP server's keymap (keyboard
+     * layout). Each definition describes which scancode corresponds to this
      * key from the perspective of the RDP server, as well as which other
      * scancodes must be pressed/released for this key to have the desired
-     * meaning. If this key does not exist within the RDP server's keymap, this
-     * will be NULL.
+     * meaning.
      */
-    const guac_rdp_keysym_desc* definition;
+    const guac_rdp_keysym_desc* definitions[GUAC_RDP_KEY_MAX_DEFINITIONS];
 
     /**
-     * The local state of this key. For the sake of simplicity, it is assumed
-     * that this state is also an accurate representation of the remote state
-     * of this key within the RDP session.
+     * The number of definitions within the definitions array. If this key does
+     * not exist within the RDP server's keymap, this will be 0.
      */
-    guac_rdp_key_state state;
+    int num_definitions;
+
+    /**
+     * The definition of this key that is currently pressed. If this key is not
+     * currently pressed, this will be NULL.
+     */
+    const guac_rdp_keysym_desc* pressed;
+
+    /**
+     * Whether this key is currently pressed by the user, and is included among
+     * the total tracked by user_pressed_keys within guac_rdp_keyboard.
+     */
+    int user_pressed;
 
 } guac_rdp_key;
 
@@ -84,7 +108,7 @@ typedef struct guac_rdp_keyboard {
      * key flags. Legal flags are KBD_SYNC_SCROLL_LOCK, KBD_SYNC_NUM_LOCK,
      * KBD_SYNC_CAPS_LOCK, and KBD_SYNC_KANA_LOCK.
      */
-    int lock_flags;
+    UINT32 lock_flags;
 
     /**
      * Whether the states of remote lock keys (Caps lock, Num lock, etc.) have
@@ -92,6 +116,11 @@ typedef struct guac_rdp_keyboard {
      */
     int synchronized;
 
+    /**
+     * The number of keys stored within the keys array.
+     */
+    unsigned int num_keys;
+
     /**
      * The local state of all keys, as well as the necessary information to
      * translate received keysyms into scancodes or sequences of scancodes for
@@ -99,13 +128,31 @@ typedef struct guac_rdp_keyboard {
      * events, while the information describing the behavior and scancode
      * mapping of each key is populated based on an associated keymap.
      *
+     * Keys within this array are in arbitrary order.
+     */
+    guac_rdp_key keys[GUAC_RDP_KEYBOARD_MAX_KEYSYMS];
+
+    /**
+     * Lookup table into the overall keys array, locating the guac_rdp_key
+     * associated with any particular keysym. If a keysym has no corresponding
+     * guac_rdp_key within the keys array, its entry within this lookuptable
+     * will be NULL.
+     *
      * The index of the key for a given keysym is determined based on a
      * simple transformation of the keysym itself. Keysyms between 0x0000 and
      * 0xFFFF inclusive are mapped to 0x00000 through 0x0FFFF, while keysyms
      * between 0x1000000 and 0x100FFFF inclusive (keysyms which are derived
      * from Unicode) are mapped to 0x10000 through 0x1FFFF.
      */
-    guac_rdp_key keys[0x20000];
+    guac_rdp_key* keys_by_keysym[0x20000];
+
+    /**
+     * The total number of keys that the user of the connection is currently
+     * holding down. This value indicates only the client-side keyboard state.
+     * It DOES NOT indicate the number of keys currently pressed within the RDP
+     * server.
+     */
+    int user_pressed_keys;
 
 } guac_rdp_keyboard;
 
@@ -158,51 +205,35 @@ void guac_rdp_keyboard_free(guac_rdp_keyboard* keyboard);
 int guac_rdp_keyboard_is_defined(guac_rdp_keyboard* keyboard, int keysym);
 
 /**
- * Sends one or more RDP key events, effectively pressing or releasing the
- * given keysym on the remote side. The key events sent will depend on the
- * current keymap. The locally-stored state of each key is remains untouched.
+ * Returns whether the key having the given keysym is currently pressed.
  *
  * @param keyboard
- *     The guac_rdp_keyboard associated with the current RDP session.
+ *     The guac_rdp_keyboard instance to check.
  *
  * @param keysym
- *     The keysym being pressed or released.
- *
- * @param pressed
- *     Zero if the keysym is being released, non-zero otherwise.
+ *     The keysym of the key being checked.
  *
  * @return
- *     Zero if the keys were successfully sent, non-zero otherwise.
+ *     Non-zero if the key is currently pressed, zero otherwise.
  */
-int guac_rdp_keyboard_send_event(guac_rdp_keyboard* keyboard,
-        int keysym, int pressed);
+int guac_rdp_keyboard_is_pressed(guac_rdp_keyboard* keyboard, int keysym);
 
 /**
- * For every keysym in the given NULL-terminated array of keysyms, send the RDP
- * key events required to update the remote state of those keys as specified,
- * depending on the current local state of those keysyms. For each key in the
- * "from" state, that key will be updated to the "to" state. The locally-stored
- * state of each key is remains untouched.
+ * Returns the local state of all known modifier keys, as a bitwise OR of the
+ * modifier flags used by the keymaps. Alternative methods of producing the
+ * effect of certain modifiers, such as holding Ctrl+Alt for AltGr when a
+ * dedicated AltGr key is unavailable, are taken into account.
+ *
+ * @see GUAC_RDP_KEYMAP_MODIFIER_SHIFT
+ * @see GUAC_RDP_KEYMAP_MODIFIER_ALTGR
  *
  * @param keyboard
  *     The guac_rdp_keyboard associated with the current RDP session.
  *
- * @param keysym_string
- *     A NULL-terminated array of keysyms, each of which will be updated.
- *
- * @param from
- *     GUAC_RDP_KEY_RELEASED if the state of currently-released keys should be
- *     updated, or GUAC_RDP_KEY_PRESSED if the state of currently-pressed keys
- *     should be updated.
- *
- * @param to 
- *     GUAC_RDP_KEY_RELEASED if the keys being updated should be marked as
- *     released, or GUAC_RDP_KEY_PRESSED if the keys being updated should be
- *     marked as pressed.
+ * @return
+ *     The local state of all known modifier keys.
  */
-void guac_rdp_keyboard_send_events(guac_rdp_keyboard* keyboard,
-        const int* keysym_string, guac_rdp_key_state from,
-        guac_rdp_key_state to);
+unsigned int guac_rdp_keyboard_get_modifier_flags(guac_rdp_keyboard* keyboard);
 
 /**
  * Updates the local state of the lock keys (such as Caps lock or Num lock),
@@ -222,7 +253,27 @@ void guac_rdp_keyboard_send_events(guac_rdp_keyboard* keyboard,
  *     KBD_SYNC_KANA_LOCK.
  */
 void guac_rdp_keyboard_update_locks(guac_rdp_keyboard* keyboard,
-        int set_flags, int clear_flags);
+        unsigned int set_flags, unsigned int clear_flags);
+
+/**
+ * Updates the local state of the modifier keys (such as Shift or AltGr),
+ * synchronizing the remote state of those keys if it is expected to differ.
+ * Valid modifier flags are defined by keymap.h.
+ *
+ * @see GUAC_RDP_KEYMAP_MODIFIER_SHIFT
+ * @see GUAC_RDP_KEYMAP_MODIFIER_ALTGR
+ *
+ * @param keyboard
+ *     The guac_rdp_keyboard associated with the current RDP session.
+ *
+ * @param set_flags
+ *     The modifier key flags which should be set.
+ *
+ * @param clear_flags
+ *     The modifier key flags which should be cleared.
+ */
+void guac_rdp_keyboard_update_modifiers(guac_rdp_keyboard* keyboard,
+        unsigned int set_flags, unsigned int clear_flags);
 
 /**
  * Updates the local state of the given keysym, sending the key events required
@@ -238,11 +289,41 @@ void guac_rdp_keyboard_update_locks(guac_rdp_keyboard* keyboard,
  * @param pressed
  *     Zero if the keysym is being released, non-zero otherwise.
  *
+ * @param source
+ *     The source of the key event represented by this call to
+ *     guac_rdp_keyboard_update_keysym().
+ *
  * @return
  *     Zero if the keys were successfully sent, non-zero otherwise.
  */
 int guac_rdp_keyboard_update_keysym(guac_rdp_keyboard* keyboard,
-        int keysym, int pressed);
+        int keysym, int pressed, guac_rdp_key_source source);
+
+/**
+ * Releases all currently pressed keys, sending key release events to the RDP
+ * server as necessary. Lock states (Caps Lock, etc.) are not affected.
+ *
+ * @param keyboard
+ *     The guac_rdp_keyboard associated with the current RDP session.
+ */
+void guac_rdp_keyboard_reset(guac_rdp_keyboard* keyboard);
+
+/**
+ * Callback which is invoked by FreeRDP when the RDP server reports changes to
+ * keyboard lock status using a Server Set Keyboard Indicators PDU.
+ *
+ * @param context
+ *     The rdpContext associated with the current RDP session.
+ *
+ * @param flags
+ *     The remote state of all lock keys, as a bitwise OR of all RDP lock key
+ *     flags. Legal flags are KBD_SYNC_SCROLL_LOCK, KBD_SYNC_NUM_LOCK,
+ *     KBD_SYNC_CAPS_LOCK, and KBD_SYNC_KANA_LOCK.
+ *
+ * @return
+ *     TRUE if successful, FALSE otherwise.
+ */
+BOOL guac_rdp_keyboard_set_indicators(rdpContext* context, UINT16 flags);
 
 #endif
 
