@@ -33,6 +33,7 @@
 #include "sftp.h"
 #endif
 
+#include <guacamole/argv.h>
 #include <guacamole/audio.h>
 #include <guacamole/client.h>
 #include <guacamole/protocol.h>
@@ -71,7 +72,7 @@ int guac_rdp_user_join_handler(guac_user* user, int argc, char** argv) {
         if (pthread_create(&rdp_client->client_thread, NULL,
                     guac_rdp_client_thread, user->client)) {
             guac_user_log(user, GUAC_LOG_ERROR,
-                    "Unable to start VNC client thread.");
+                    "Unable to start RDP client thread.");
             return 1;
         }
 
@@ -100,10 +101,13 @@ int guac_rdp_user_join_handler(guac_user* user, int argc, char** argv) {
     /* Only handle events if not read-only */
     if (!settings->read_only) {
 
-        /* General mouse/keyboard/clipboard events */
-        user->mouse_handler     = guac_rdp_user_mouse_handler;
-        user->key_handler       = guac_rdp_user_key_handler;
-        user->clipboard_handler = guac_rdp_clipboard_handler;
+        /* General mouse/keyboard events */
+        user->mouse_handler = guac_rdp_user_mouse_handler;
+        user->key_handler = guac_rdp_user_key_handler;
+
+        /* Inbound (client to server) clipboard transfer */
+        if (!settings->disable_paste)
+            user->clipboard_handler = guac_rdp_clipboard_handler;
 
         /* Display size change events */
         user->size_handler = guac_rdp_user_size_handler;
@@ -113,6 +117,10 @@ int guac_rdp_user_join_handler(guac_user* user, int argc, char** argv) {
 
         /* Inbound arbitrary named pipes */
         user->pipe_handler = guac_rdp_pipe_svc_pipe_handler;
+        
+        /* If we own it, register handler for updating parameters during connection. */
+        if (user->owner)
+            user->argv_handler = guac_argv_handler;
 
     }
 
@@ -124,20 +132,21 @@ int guac_rdp_user_file_handler(guac_user* user, guac_stream* stream,
         char* mimetype, char* filename) {
 
     guac_rdp_client* rdp_client = (guac_rdp_client*) user->client->data;
-
-#ifdef ENABLE_COMMON_SSH
     guac_rdp_settings* settings = rdp_client->settings;
 
-    /* If SFTP is enabled, it should be used for default uploads only if RDPDR
-     * is not enabled or its upload directory has been set */
-    if (rdp_client->sftp_filesystem != NULL) {
+#ifdef ENABLE_COMMON_SSH
+
+    /* If SFTP is enabled and SFTP uploads have not been disabled, it should be
+     * used for default uploads only if RDPDR is not enabled or its upload
+     * directory has been set */
+    if (rdp_client->sftp_filesystem != NULL && !settings->sftp_disable_upload) {
         if (!settings->drive_enabled || settings->sftp_directory != NULL)
             return guac_rdp_sftp_file_handler(user, stream, mimetype, filename);
     }
 #endif
 
     /* Default to using RDPDR uploads (if enabled) */
-    if (rdp_client->filesystem != NULL)
+    if (rdp_client->filesystem != NULL && !settings->disable_upload)
         return guac_rdp_upload_file_handler(user, stream, mimetype, filename);
 
     /* File transfer not enabled */
