@@ -171,6 +171,20 @@ typedef struct guac_telnet_settings {
      */
     int resolution;
 
+    /**
+     * Whether outbound clipboard access should be blocked. If set, it will not
+     * be possible to copy data from the terminal to the client using the
+     * clipboard.
+     */
+    bool disable_copy;
+
+    /**
+     * Whether inbound clipboard access should be blocked. If set, it will not
+     * be possible to paste data from the client to the terminal using the
+     * clipboard.
+     */
+    bool disable_paste;
+
     /**
      * The path in which the typescript should be saved, if enabled. If no
      * typescript should be saved, this will be NULL.
@@ -242,6 +256,30 @@ typedef struct guac_telnet_settings {
      * The terminal emulator type that is passed to the remote system.
      */
     char* terminal_type;
+    
+    /**
+     * Whether or not to send the magic Wake-on-LAN (WoL) packet prior to
+     * continuing the connection.
+     */
+    bool wol_send_packet;
+    
+    /**
+     * The MAC address to put in the magic WoL packet for the remote host to
+     * wake.
+     */
+    char* wol_mac_addr;
+    
+    /**
+     * The broadcast address to which to send the magic WoL packet to wake
+     * the remote host.
+     */
+    char* wol_broadcast_addr;
+    
+    /**
+     * The number of seconds to wait after sending the magic WoL packet before
+     * continuing the connection.
+     */
+    int wol_wait_time;
 
 } guac_telnet_settings;
 
